<?php

/*********************************************************************************

    Author ......... johnrembo
    Contact ........ johnrembo@cacti-forums
    Program ........ collects poller output in cacti's mysql database

**********************************************************************************/

function plugin_init_transporter() {
    global $plugin_hooks;
    $plugin_hooks['config_arrays']['transporter'] = 'transporter_config_arrays';
    $plugin_hooks['config_settings']['transporter'] = 'transporter_config_settings';
    $plugin_hooks['poller_output']['transporter'] = 'transporter_poller_output';
    $plugin_hooks['graph_buttons']['transporter'] = 'transporter_graph_buttons';
}

function transporter_version () {
    return array( 
    		'name' 		=> 'transporter',
		'version' 	=> '0.2',
		'longname'	=> 'collects poller output in cacti\'s mysql database',
		'author'	=> 'johnrembo',
		'homepage'	=> '',
		'email'		=> 'johnrembo@cacti-forums',
		'url'		=> ''
		);
}

function transporter_config_arrays () {
    $realm_id=102;
    global $user_auth_realms, $user_auth_realm_filenames;
    $user_auth_realms[$realm_id]='Transporter statistics';
    $user_auth_realm_filenames['node_stats.php'] = $realm_id;
}

function transporter_config_settings () {
    global $tabs, $settings;

    $tabs["transporter"] = "Transporter";

    $v = transporter_version();

    $temp = array(
	    "transporter_header"	=>	array(
						"friendly_name" => "Transporter v".$v['version']." configuration",
						"method" => "spacer"
					     ),
	    "transporter_enable"	=>	array(
						"friendly_name" => "Enable transporter",
						"description" => "By checking this you will enable transporter plugin",
						"default" => "on",
						"method" => "checkbox"
					     ),
	    "transporter_header2"	=>	array(
						"friendly_name" => "Data-output settings",
						"method" => "spacer"
					     ),
    	    "transporter_dbtable" 	=> 	array(
	                    			    "friendly_name" => "Output table",
				            	    "description" => "Table name to output data to",
						    "method" => "textbox",
						    "max_length" => "255",
						    "size"	=> "15",
						    "default" => "plugin_transporter_poller_output_archive"
					     ),
    	    "transporter_dbtable_type" 	=> 	array(
	                    			    "friendly_name" => "Table type",
				            	    "description" => "use MyISAM type for local storage(slower) or MEMORY type for RAM storage(much faster, but consumes memory)<br>You should consider increasing max_heap_table_size=xxxxxxx in my.cnf if MEMORY storage type is chosen",
						    "method" => "drop_array",
						    "default" => "0",
						    "array" => array("MyISAM","MEMORY")
					     ),
    	    "transporter_dbhistcycles" 	=> 	array(
	                    			    "friendly_name" => "History cycles",
				            	    "description" => "Number of output records to save for each local_data_id. 1 cycle=1 Update Interval (5 min. usualy). Recommended value >=2<br>0=unlimited, 12=1hour, 288=24h etc....",
						    "method" => "textbox",
						    "max_length" => "6",
						    "size"	=> "5",
						    "default" => "12"
					     ),
            "transporter_debug"		=>	array(
                                                    "friendly_name" => "DEBUG",
                                                    "description" => "Choose YES to display info about skipped empty records in cacti log",
                                                    "method" => "drop_array",
                                                    "default" => 0,
                                                    "array" => array("yes","no")
					    ),
            "transporter_sample_button"	=>	array(
                                                    "friendly_name" => "Sample button",
                                                    "description" => "Show/Hide \"transporter\" button next to graph for statistical information demo",
                                                    "method" => "drop_array",
                                                    "default" => 0,
                                                    "array" => array("no","yes")
					    )
		);
	
	if (isset($settings["transporter"]))
		$settings["transporter"] = array_merge($settings["transporter"], $temp);
	else
		$settings["transporter"]=$temp;
}

function transporter_poller_output ($rrd_update_array) {
    global $config, $database_default,$debug;

    if ( read_config_option('transporter_enable') != 'on' ) return;

    $T = array ('dbtable','dbhistcycles');
    foreach ($T as $t) {
	eval("\$$t=read_config_option('transporter_$t');");
    }

    $dbname=$database_default;
    $debug = (read_config_option('transporter_debug')=="0") ? true : false;
    
    transporter_init($dbname,$dbtable);
    transporter_process($dbtable,$dbhistcycles,&$rrd_update_array);
}

function transporter_init ($dbname,$dbtable) {
        global $config,$debug;
        include_once($config["library_path"] . "/database.php");

        $sql = "show tables from $dbname where Tables_in_$dbname='$dbtable'";

        $result = db_fetch_row($sql);

	$sql=array();
	
        if (empty($result)) {
	    $type=read_config_option('transporter_dbtable_type');
	    if($type==1) {
		$sql="CREATE TABLE `$dbtable` (
			`id` int(10) unsigned NOT NULL auto_increment,
			`local_data_id` int(10) unsigned NOT NULL,
			`timestamp` datetime NOT NULL,
			`poller_output` varchar(255) NOT NULL,
			PRIMARY KEY  USING HASH (`id`),
			KEY `$dbtable_index01` (`local_data_id`,`timestamp`)
		    ) ENGINE=MEMORY AVG_ROW_LENGTH=50 ROW_FORMAT=DYNAMIC;";
	    } else {
                $sql = "CREATE TABLE `$dbtable` (
			`id` INTEGER UNSIGNED NOT NULL DEFAULT NULL AUTO_INCREMENT,
                        `local_data_id` INTEGER UNSIGNED NOT NULL,
                        `timestamp` DATETIME NOT NULL DEFAULT '0000-00-00 00:00',
                        `poller_output` TEXT NOT NULL,
                        PRIMARY KEY (`id`)
                        ) TYPE=MyISAM;";
	    }

    	    $result = db_execute($sql);
	    if (!$result && $debug) {
		cacti_log("TRANSPORTER: unable to create table \"$dbtable\"", TRUE, "TRANSPORTER");
		return;
	    } else {
		if ($debug) cacti_log("TRANSPORTER: table \"$dbtable\" of type ".(($type==1) ? 'MEMORY' : 'MyISAM')." created", TRUE, "TRANSPORTER");
	    }
    }
}

function transporter_graph_buttons($params) {
    $lgi=$params[1]['local_graph_id'];
    if (read_config_option("transporter_sample_button")==1) {
        echo '<a href=# onClick=\'javascript:wnd=window.open("plugins/transporter/node_stats.php?local_graph_id='.$lgi.'&action=showtable","transporter","menubar=no,toolbar=no,resizable=yes,scrollbars=yes,width=510,height=480");\'>
    		<img border="0" alt="Transporter Sample Data" title="Transporter Sample Data" style="padding: 3px" src="plugins/transporter/chart.png"></a>';
    }
}

function transporter_process($dbtable,$hcycles,$data) {
    global $config,$debug;

    $sql[]="DELETE
		$dbtable.*
	    FROM
		$dbtable
	    LEFT JOIN 
		data_template_data dtd USING (local_data_id)
	    WHERE
		(UNIX_TIMESTAMP(NOW()) - UNIX_TIMESTAMP($dbtable.timestamp))/dtd.rrd_step > $hcycles";

    if (empty($data))
	if ($debug) cacti_log("TRANSPORTER: wrong or no data", TURE, "TRANSPORTER");
	
    foreach ($data as $item) {
	$ldi=$item['local_data_id'];
	$output = '';
	foreach ($item['times'] as $timestamp => $itemm) {
	    foreach ($itemm as $var => $val)
		$output .= (($var!='') ? "$var:" : "").$val." ";

	$output = trim($output);
	$sql[]="INSERT INTO 
		    $dbtable (local_data_id,timestamp,poller_output) 
		VALUES 
		    ($ldi,FROM_UNIXTIME($timestamp),\"$output\")";
	}
    }
    
    if (!empty($sql)) {
	for ($a = 0; $a < count($sql); $a++) {
            $result = db_execute($sql[$a]);
	    if ($result) $ok++;
	}

    if ($debug) cacti_log("TRANSPORTER: $ok of ".count($sql)." record(s) processed", TRUE, "TRANSPORTER");
    }
}
?>
